@@ -68,13 +68,21 @@ const pitrCheckPeriod = time.Second * 15
 func (a *Agent) PITR() {
 	a.log.Printf("starting PITR routine")
 
-	tk := time.NewTicker(pitrCheckPeriod)
-	defer tk.Stop()
-	for range tk.C {
+	for {
+		wait := pitrCheckPeriod
+
 		err := a.pitr()
 		if err != nil {
-			a.log.Error(pbm.CmdPITR, "", "%v", err)
+			// wee need epoch just to log pitr err with an extra context
+			// so not much care if we get it or not
+			ep, _ := a.pbm.GetEpoch()
+			a.log.Error(string(pbm.CmdPITR), "", "", ep.TS(), "init: %v", err)
+
+			// penalty to the failed node so healthy nodes would have priority on next try
+			wait *= 2
 		}
+
+		time.Sleep(wait)
 	}
 }
 
@@ -97,6 +105,22 @@ func (a *Agent) pitr() (err error) {
 		return nil
 	}
 
+	// just a check before a real locking
+	// just trying to avoid redundant heavy operations
+	moveOn, err := a.pitrLockCheck()
+	if err != nil {
+		return errors.Wrap(err, "check if already run")
+	}
+
+	if !moveOn {
+		return nil
+	}
+
+	// shoud be after the lock pre-check
+	//
+	// if node failing, then some other agent with healthy node will hopefully catchup
+	// so this code won't be reached and will not pollute log with "pitr" errors while
+	// the other node does successfully slice
 	ninf, err := a.node.GetInfo()
 	if err != nil {
 		return errors.Wrap(err, "get node info")
@@ -111,18 +135,6 @@ func (a *Agent) pitr() (err error) {
 		return nil
 	}
 
-	// just a check before a real locking
-	// just trying to avoid redundant heavy operations
-	ts, err := a.pbm.ClusterTime()
-	if err != nil {
-		return errors.Wrap(err, "read cluster time")
-	}
-	tl, err := a.pbm.GetLockData(&pbm.LockHeader{Replset: a.node.RS()})
-	// ErrNoDocuments or stale lock the only reasons to continue
-	if err != mongo.ErrNoDocuments && tl.Heartbeat.T+pbm.StaleFrameSec >= ts.T {
-		return errors.Wrap(err, "check if already run")
-	}
-
 	ibcp := pitr.NewBackup(a.node.RS(), a.pbm, a.node)
 
 	err = ibcp.Catchup()
@@ -130,33 +142,35 @@ func (a *Agent) pitr() (err error) {
 		return errors.Wrap(err, "defining starting point for the backup")
 	}
 
-	stg, err := a.pbm.GetStorage()
+	ep, err := a.pbm.GetEpoch()
+	if err != nil {
+		return errors.Wrap(err, "get epoch")
+	}
+	l := a.log.NewEvent(string(pbm.CmdPITR), "", "", ep.TS())
+
+	stg, err := a.pbm.GetStorage(l)
 	if err != nil {
 		return errors.Wrap(err, "unable to get storage configuration")
 	}
 
+	epts := ep.TS()
 	lock := a.pbm.NewLock(pbm.LockHeader{
 		Replset: a.node.RS(),
 		Node:    a.node.Name(),
 		Type:    pbm.CmdPITR,
+		Epoch:   &epts,
 	})
 
-	got, err := a.aquireLock(lock, nil)
+	got, err := a.aquireLock(lock)
 	if err != nil {
 		return errors.Wrap(err, "acquiring lock")
 	}
 	if !got {
+		l.Debug("skip: lock not acquired")
 		return nil
 	}
 
 	go func() {
-		defer func() {
-			err := lock.Release()
-			if err != nil {
-				a.log.Error(pbm.CmdPITR, "", "release lock: %v", err)
-			}
-		}()
-
 		ctx, cancel := context.WithCancel(context.Background())
 		w := make(chan struct{})
 
@@ -165,63 +179,109 @@ func (a *Agent) pitr() (err error) {
 			wakeup: w,
 		})
 
-		err := ibcp.Stream(ctx, w, stg, pbm.CompressionTypeS2)
-		if err != nil {
-			switch err.(type) {
+		streamErr := ibcp.Stream(ctx, ep, w, stg, pbm.CompressionTypeS2)
+		if streamErr != nil {
+			switch streamErr.(type) {
 			case pitr.ErrOpMoved:
-				a.log.Info(pbm.CmdPITR, "", "streaming oplog: %v", err)
+				l.Info("streaming oplog: %v", streamErr)
 			default:
-				a.log.Error(pbm.CmdPITR, "", "streaming oplog: %v", err)
+				l.Error("streaming oplog: %v", streamErr)
 			}
 		}
 
+		if err := lock.Release(); err != nil {
+			l.Error("release lock: %v", err)
+		}
+
+		// Penalty to the failed node so healthy nodes would have priority on next try.
+		// But lock has to be released first. Otherwise, healthy nodes would wait for the lock release
+		// and the penalty won't have any sense.
+		if streamErr != nil {
+			time.Sleep(pitrCheckPeriod * 2)
+		}
+
 		a.unsetPitr()
 	}()
 
 	return nil
 }
 
+func (a *Agent) pitrLockCheck() (moveOn bool, err error) {
+	ts, err := a.pbm.ClusterTime()
+	if err != nil {
+		return false, errors.Wrap(err, "read cluster time")
+	}
+
+	tl, err := a.pbm.GetLockData(&pbm.LockHeader{Replset: a.node.RS()})
+	if err != nil {
+		// mongo.ErrNoDocuments means no lock and we're good to move on
+		if err == mongo.ErrNoDocuments {
+			return true, nil
+		}
+
+		return false, errors.Wrap(err, "get lock")
+	}
+
+	// stale lock means we should move on and clean it up during the lock.Aquire
+	return tl.Heartbeat.T+pbm.StaleFrameSec < ts.T, nil
+}
+
 // PITRestore starts the point-in-time recovery
-func (a *Agent) PITRestore(r pbm.PITRestoreCmd) {
-	tsstr := time.Unix(r.TS, 0).UTC().Format(time.RFC3339)
+func (a *Agent) PITRestore(r pbm.PITRestoreCmd, opid pbm.OPID, ep pbm.Epoch) {
+	l := a.log.NewEvent(string(pbm.CmdPITR), time.Unix(r.TS, 0).UTC().Format(time.RFC3339), opid.String(), ep.TS())
+
 	nodeInfo, err := a.node.GetInfo()
 	if err != nil {
-		a.log.Error(pbm.CmdPITR, tsstr, "get node info: %v", err)
+		l.Error("get node info: %v", err)
 		return
 	}
 	if !nodeInfo.IsPrimary {
-		a.log.Info(pbm.CmdPITR, tsstr, "Node in not suitable for restore")
+		l.Info("Node in not suitable for restore")
 		return
 	}
 
+	epts := ep.TS()
 	lock := a.pbm.NewLock(pbm.LockHeader{
 		Type:    pbm.CmdPITRestore,
 		Replset: nodeInfo.SetName,
 		Node:    nodeInfo.Me,
+		OPID:    opid.String(),
+		Epoch:   &epts,
 	})
 
-	got, err := lock.Acquire()
+	got, err := a.aquireLock(lock)
 	if err != nil {
-		a.log.Error(pbm.CmdPITR, tsstr, "acquiring lock: %v", err)
+		l.Error("acquiring lock: %v", err)
 		return
 	}
 	if !got {
-		a.log.Error(pbm.CmdPITR, tsstr, "unbale to run the restore while another backup or restore process running")
+		l.Debug("skip: lock not acquired")
+		l.Error("unbale to run the restore while another backup or restore process running")
 		return
 	}
 
 	defer func() {
 		err := lock.Release()
 		if err != nil {
-			a.log.Error(pbm.CmdPITR, tsstr, "release lock: %v", err)
+			l.Error("release lock: %v", err)
 		}
 	}()
 
-	a.log.Info(pbm.CmdPITR, tsstr, "recovery started")
-	err = restore.New(a.pbm, a.node).PITR(r)
+	l.Info("recovery started")
+	err = restore.New(a.pbm, a.node).PITR(r, opid, l)
 	if err != nil {
-		a.log.Error(pbm.CmdPITR, tsstr, "restore: %v", err)
+		l.Error("restore: %v", err)
 		return
 	}
-	a.log.Info(pbm.CmdPITR, tsstr, "recovery successfully finished")
+	l.Info("recovery successfully finished")
+
+	if nodeInfo.IsLeader() {
+		epch, err := a.pbm.ResetEpoch()
+		if err != nil {
+			l.Error("reset epoch")
+			return
+		}
+
+		l.Debug("epoch set to %v", epch)
+	}
 }
