@@ -46,10 +46,12 @@ func (a *Agent) CancelBackup() {
 }
 
 // Backup starts backup
-func (a *Agent) Backup(bcp pbm.BackupCmd) {
+func (a *Agent) Backup(bcp pbm.BackupCmd, opid pbm.OPID, ep pbm.Epoch) {
+	l := a.log.NewEvent(string(pbm.CmdBackup), bcp.Name, opid.String(), ep.TS())
+
 	nodeInfo, err := a.node.GetInfo()
 	if err != nil {
-		a.log.Error(pbm.CmdBackup, bcp.Name, "get node info: %v", err)
+		l.Error("get node info: %v", err)
 		return
 	}
 
@@ -63,20 +65,20 @@ func (a *Agent) Backup(bcp pbm.BackupCmd) {
 	if nodeInfo.IsPrimary && nodeInfo.Me == nodeInfo.Primary {
 		err = a.node.DropTMPcoll()
 		if err != nil {
-			a.log.Error(pbm.CmdBackup, bcp.Name, "ensure no tmp collections: %v", err)
+			l.Error("ensure no tmp collections: %v", err)
 			return
 		}
 	}
 
 	q, err := backup.NodeSuits(a.node, nodeInfo)
 	if err != nil {
-		a.log.Error(pbm.CmdBackup, bcp.Name, "node check: %v", err)
+		l.Error("node check: %v", err)
 		return
 	}
 
 	// node is not suitable for doing backup
 	if !q {
-		a.log.Info(pbm.CmdBackup, bcp.Name, "node in not suitable for backup")
+		l.Info("node is not suitable for backup")
 		return
 	}
 
@@ -98,25 +100,27 @@ func (a *Agent) Backup(bcp pbm.BackupCmd) {
 		Replset: nodeInfo.SetName,
 	}).Release()
 	if err != nil {
-		a.log.Warning(pbm.CmdBackup, bcp.Name, "clearing pitr locks: %v", err)
+		l.Warning("clearing pitr locks: %v", err)
 	}
 	// wakeup the slicer not to wait for the tick
 	a.wakeupPitr()
 
+	epts := ep.TS()
 	lock := a.pbm.NewLock(pbm.LockHeader{
-		Type:       pbm.CmdBackup,
-		Replset:    nodeInfo.SetName,
-		Node:       nodeInfo.Me,
-		BackupName: bcp.Name,
+		Type:    pbm.CmdBackup,
+		Replset: nodeInfo.SetName,
+		Node:    nodeInfo.Me,
+		OPID:    opid.String(),
+		Epoch:   &epts,
 	})
 
-	got, err := a.aquireLock(lock, a.pbm.MarkBcpStale)
+	got, err := a.aquireLock(lock)
 	if err != nil {
-		a.log.Error(pbm.CmdBackup, bcp.Name, "acquiring lock: %v", err)
+		l.Error("acquiring lock: %v", err)
 		return
 	}
 	if !got {
-		a.log.Info(pbm.CmdBackup, bcp.Name, "backup has been scheduled on another replset node")
+		l.Debug("skip: lock not acquired")
 		return
 	}
 
@@ -125,91 +129,96 @@ func (a *Agent) Backup(bcp pbm.BackupCmd) {
 		header: &bcp,
 		cancel: cancel,
 	})
-	a.log.Info(pbm.CmdBackup, bcp.Name, "backup started")
-	tstart := time.Now()
-	err = backup.New(ctx, a.pbm, a.node).Run(bcp)
+	l.Info("backup started")
+	err = backup.New(ctx, a.pbm, a.node).Run(bcp, opid, l)
 	a.unsetBcp()
 	if err != nil {
 		if errors.Is(err, backup.ErrCancelled) {
-			a.log.Info(pbm.CmdBackup, bcp.Name, "backup was canceled")
+			l.Info("backup was canceled")
 		} else {
-			a.log.Error(pbm.CmdBackup, bcp.Name, "backup: %v", err)
+			l.Error("backup: %v", err)
 		}
 	} else {
-		a.log.Info(pbm.CmdBackup, bcp.Name, "backup finished")
+		l.Info("backup finished")
+
+		// Update PITR "changed" option to "reset" observation by pbm list of
+		// any PITR related errors in the log since some of the errors might
+		// be fixed by the backup. If not (errors wasn't fixed by bcp) PITR will
+		// generate new errors so we won't miss anything
+		if nodeInfo.IsLeader() {
+			epch, err := a.pbm.ResetEpoch()
+			if err != nil {
+				l.Error("reset epoch")
+			} else {
+				l.Debug("epoch set to %v", epch)
+			}
+		}
 	}
 
-	// Update PITR "changed" option to "reset" observation by pbm list of
-	// any PITR related errors in the log since some of the errors might
-	// be fixed by the backup. If not (errors wasn't fixed by bcp) PITR will
-	// generate new errors so we won't miss anything
-	err = a.pbm.ConfigBumpPITRepoch()
-	if err != nil {
-		a.log.Warning(pbm.CmdBackup, bcp.Name, "update PITR obesrvation ts")
-	}
-
-	// In the case of fast backup (small db) we have to wait before releasing the lock.
-	// Otherwise, since the primary node waits for `WaitBackupStart*0.9` before trying to acquire the lock
-	// it might happen that the backup will be made twice:
-	//
-	// secondary1 >---------*!lock(fail - acuired by s1)---------------------------
-	// secondary2 >------*lock====backup====*unlock--------------------------------
-	// primary    >--------*wait--------------------*lock====backup====*unlock-----
-	//
-	// Secondaries also may start trying to acquire a lock with quite an interval (e.g. due to network issues)
-	// TODO: we cannot rely on the nodes wall clock.
-	// TODO: ? pbmBackups should have unique index by name ?
-	needToWait := pbm.WaitBackupStart - time.Since(tstart)
-	if needToWait > 0 {
-		time.Sleep(needToWait)
-	}
+	l.Debug("releasing lock")
 	err = lock.Release()
 	if err != nil {
-		a.log.Error(pbm.CmdBackup, bcp.Name, "unable to release backup lock %v: %v", lock, err)
+		l.Error("unable to release backup lock %v: %v", lock, err)
 	}
 }
 
 // Restore starts the restore
-func (a *Agent) Restore(r pbm.RestoreCmd) {
+func (a *Agent) Restore(r pbm.RestoreCmd, opid pbm.OPID, ep pbm.Epoch) {
+	l := a.log.NewEvent(string(pbm.CmdRestore), r.BackupName, opid.String(), ep.TS())
+
 	nodeInfo, err := a.node.GetInfo()
 	if err != nil {
-		a.log.Error(pbm.CmdRestore, r.BackupName, "get node info: %v", err)
+		l.Error("get node info: %v", err)
 		return
 	}
 	if !nodeInfo.IsPrimary {
-		a.log.Info(pbm.CmdRestore, r.BackupName, "node in not suitable for restore")
+		l.Info("node is not primary so it unsuitable to do restore")
 		return
 	}
 
+	epts := ep.TS()
 	lock := a.pbm.NewLock(pbm.LockHeader{
-		Type:       pbm.CmdRestore,
-		Replset:    nodeInfo.SetName,
-		Node:       nodeInfo.Me,
-		BackupName: r.Name,
+		Type:    pbm.CmdRestore,
+		Replset: nodeInfo.SetName,
+		Node:    nodeInfo.Me,
+		OPID:    opid.String(),
+		Epoch:   &epts,
 	})
 
-	got, err := lock.Acquire()
+	got, err := a.aquireLock(lock)
 	if err != nil {
-		a.log.Error(pbm.CmdRestore, r.BackupName, "acquiring lock: %v", err)
+		l.Error("acquiring lock: %v", err)
 		return
 	}
 	if !got {
-		a.log.Error(pbm.CmdRestore, r.BackupName, "unbale to run the restore while another backup or restore process running")
+		l.Debug("skip: lock not acquired")
+		l.Error("unbale to run the restore while another operation running")
 		return
 	}
 
 	defer func() {
+		l.Debug("releasing lock")
 		err := lock.Release()
 		if err != nil {
-			a.log.Error(pbm.CmdRestore, r.BackupName, "release lock: %v", err)
+			l.Error("release lock: %v", err)
 		}
 	}()
 
-	a.log.Info(pbm.CmdRestore, r.BackupName, "restore started")
-	err = restore.New(a.pbm, a.node).Snapshot(r)
+	l.Info("restore started")
+	err = restore.New(a.pbm, a.node).Snapshot(r, opid, l)
 	if err != nil {
-		a.log.Error(pbm.CmdRestore, r.BackupName, "restore: %v", err)
+		l.Error("restore: %v", err)
 		return
 	}
-	a.log.Info(pbm.CmdRestore, r.BackupName, "restore finished successfully")
+	l.Info("restore finished successfully")
+
+	if nodeInfo.IsLeader() {
+		epch, err := a.pbm.ResetEpoch()
+		if err != nil {
+			l.Error("reset epoch")
+			return
+		}
+
+		l.Debug("epoch set to %v", epch)
+	}
 }
