@@ -11,6 +11,7 @@ import (
 	"go.mongodb.org/mongo-driver/mongo"
 
 	"github.com/percona/percona-backup-mongodb/pbm"
+	plog "github.com/percona/percona-backup-mongodb/pbm/log"
 )
 
 func backup(cn *pbm.PBM, bcpName, compression string) (string, error) {
@@ -28,13 +29,13 @@ func backup(cn *pbm.PBM, bcpName, compression string) (string, error) {
 	// But if there is some stale lock leave it for agents to deal with.
 	for _, l := range locks {
 		if l.Heartbeat.T+pbm.StaleFrameSec >= ts.T && l.Type != pbm.CmdPITR {
-			return "", errors.Errorf("another operation in progress, %s/%s [%s/%s]", l.Type, l.BackupName, l.Replset, l.Node)
+			return "", errors.Errorf("another operation in progress, %s/%s [%s/%s]", l.Type, l.OPID, l.Replset, l.Node)
 		}
 	}
 
 	cfg, err := cn.GetConfig()
 	if err != nil {
-		if err == mongo.ErrNoDocuments {
+		if errors.Is(err, mongo.ErrNoDocuments) {
 			return "", errors.New("no store set. Set remote store with <pbm store set>")
 		}
 		return "", errors.Wrap(err, "get remote-store")
@@ -126,22 +127,12 @@ func printBackupList(cn *pbm.PBM, size int64) {
 	fmt.Println("Backup snapshots:")
 	for i := len(bcps) - 1; i >= 0; i-- {
 		b := bcps[i]
-		var bcp string
-		switch b.Status {
-		case pbm.StatusDone:
-			bcp = b.Name
-		case pbm.StatusCancelled:
-			bcp = fmt.Sprintf("%s\tCancelled at %s", b.Name, time.Unix(b.LastTransitionTS, 0).UTC().Format(time.RFC3339))
-		case pbm.StatusError:
-			bcp = fmt.Sprintf("%s\tFailed with \"%s\"", b.Name, b.Error)
-		default:
-			bcp, err = printBackupProgress(b, cn)
-			if err != nil {
-				log.Fatalf("Error: list backup %s: %v\n", b.Name, err)
-			}
+
+		if b.Status != pbm.StatusDone {
+			continue
 		}
 
-		fmt.Println(" ", bcp)
+		fmt.Printf("  %s [complete: %s]\n", b.Name, fmtTS(int64(b.LastWriteTS.T)))
 	}
 }
 
@@ -156,7 +147,7 @@ func printPITR(cn *pbm.PBM, size int, full bool) {
 		log.Fatalf("Error: define cluster state: %v", err)
 	}
 
-	shards := []pbm.Shard{{ID: inf.SetName}}
+	shards := []pbm.Shard{{RS: inf.SetName}}
 	if inf.IsSharded() {
 		s, err := cn.GetShards()
 		if err != nil {
@@ -165,43 +156,13 @@ func printPITR(cn *pbm.PBM, size int, full bool) {
 		shards = append(shards, s...)
 	}
 
-	ts, err := cn.ClusterTime()
-	if err != nil {
-		log.Fatalf("Error: read cluster time: %v", err)
-	}
-
-	cfg, err := cn.GetConfig()
-	if err != nil {
-		if errors.Is(err, mongo.ErrNoDocuments) {
-			return
-		}
-
-		log.Fatalf("Error: read config: %v", cfg)
-	}
-
 	now := time.Now().Unix()
-	var pitrList, pitrErrors string
+	var pitrList string
 	var rstlines [][]pbm.Timeline
 	for _, s := range shards {
-		if on {
-			err := pitrState(cn, s.ID, ts)
-			if err == errPITRBackup && cfg.PITR.Changed <= time.Now().Add(time.Minute*-1).Unix() {
-				pitrErrors += fmt.Sprintf("  %s: PITR backup didn't started\n", s.ID)
-			} else if err != nil {
-				log.Printf("Error: check PITR state for shard '%s': %v", s.ID, err)
-			}
-			lg, err := pitrLog(cn, s.ID, cfg.PITR.Changed)
-			if err != nil {
-				log.Printf("Error: get log for shard '%s': %v", s.ID, err)
-			}
-			if lg != "" {
-				pitrErrors += fmt.Sprintf("  %s: %s\n", s.ID, lg)
-			}
-		}
-
-		tlns, err := cn.PITRGetValidTimelines(s.ID, now)
+		tlns, err := cn.PITRGetValidTimelines(s.RS, now, nil)
 		if err != nil {
-			log.Printf("Error: get PITR timelines for %s replset: %v", s.ID, err)
+			log.Printf("Error: get PITR timelines for %s replset: %v", s.RS, err)
 		}
 
 		if len(tlns) == 0 {
@@ -213,7 +174,7 @@ func printPITR(cn *pbm.PBM, size int, full bool) {
 		}
 
 		if full {
-			rsout := fmt.Sprintf("  %s:", s.ID)
+			rsout := fmt.Sprintf("  %s:", s.RS)
 			for _, tln := range tlns {
 				rsout += fmt.Sprintf(" %v,", tln)
 			}
@@ -227,21 +188,16 @@ func printPITR(cn *pbm.PBM, size int, full bool) {
 	}
 
 	if on {
-		fmt.Println("PITR <on>:")
+		fmt.Println("\nPITR <on>:")
 	}
 	if len(rstlines) > 0 && len(rstlines) == len(shards) {
 		if !on {
-			fmt.Println("PITR <off>:")
+			fmt.Println("\nPITR <off>:")
 		}
 		for _, tl := range pbm.MergeTimelines(rstlines...) {
 			fmt.Println(" ", tl)
 		}
 	}
-
-	if len(pitrErrors) > 0 {
-		fmt.Printf("\n!Failed to run PITR backup. Agent logs:\n%s", pitrErrors)
-	}
-
 }
 
 var errPITRBackup = errors.New("PITR backup failed")
@@ -267,8 +223,17 @@ func pitrState(cn *pbm.PBM, rs string, ts primitive.Timestamp) error {
 	return nil
 }
 
-func pitrLog(cn *pbm.PBM, rs string, after int64) (string, error) {
-	l, err := cn.LogGet(rs, pbm.TypeError, pbm.CmdPITR, 1)
+func pitrLog(cn *pbm.PBM, rs string, e pbm.Epoch) (string, error) {
+	l, err := cn.LogGet(
+		&plog.LogRequest{
+			LogKeys: plog.LogKeys{
+				RS:       rs,
+				Severity: plog.Error,
+				Event:    string(pbm.CmdPITR),
+				Epoch:    e.TS(),
+			},
+		},
+		1)
 	if err != nil {
 		return "", errors.Wrap(err, "get log records")
 	}
@@ -276,17 +241,13 @@ func pitrLog(cn *pbm.PBM, rs string, after int64) (string, error) {
 		return "", nil
 	}
 
-	if l[0].TS < after {
-		return "", nil
-	}
-
 	return l[0].String(), nil
 }
 
 func printBackupProgress(b pbm.BackupMeta, pbmClient *pbm.PBM) (string, error) {
 	locks, err := pbmClient.GetLocks(&pbm.LockHeader{
-		Type:       pbm.CmdBackup,
-		BackupName: b.Name,
+		Type: pbm.CmdBackup,
+		OPID: b.OPID,
 	})
 
 	if err != nil {
