@@ -7,7 +7,6 @@ import (
 	"fmt"
 	"io"
 	"log"
-	"runtime"
 	"strings"
 	"time"
 
@@ -21,6 +20,7 @@ import (
 	"go.mongodb.org/mongo-driver/mongo"
 
 	"github.com/percona/percona-backup-mongodb/pbm"
+	plog "github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
 	"github.com/percona/percona-backup-mongodb/pbm/storage/s3"
 	"github.com/percona/percona-backup-mongodb/version"
@@ -31,13 +31,14 @@ func init() {
 	//
 	// duplicated in backup/restore packages just
 	// in the sake of clarity
-	mlog.SetDateFormat(pbm.LogTimeFormat)
+	mlog.SetDateFormat(plog.LogTimeFormat)
 }
 
 type Backup struct {
 	cn   *pbm.PBM
 	node *pbm.Node
 	ctx  context.Context
+	opid string
 }
 
 func New(ctx context.Context, cn *pbm.PBM, node *pbm.Node) *Backup {
@@ -49,8 +50,8 @@ func New(ctx context.Context, cn *pbm.PBM, node *pbm.Node) *Backup {
 }
 
 // Run runs the backup
-func (b *Backup) Run(bcp pbm.BackupCmd) (err error) {
-	return b.run(bcp)
+func (b *Backup) Run(bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error) {
+	return b.run(bcp, opid, l)
 }
 
 type errMetaExists struct {
@@ -65,13 +66,15 @@ func (e *errMetaExists) Unwrap() error { return e.error }
 
 // run the backup.
 // TODO: describe flow
-func (b *Backup) run(bcp pbm.BackupCmd) (err error) {
+func (b *Backup) run(bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error) {
 	inf, err := b.node.GetInfo()
 	if err != nil {
 		return errors.Wrap(err, "get cluster info")
 	}
 
+	b.opid = opid.String()
 	meta := &pbm.BackupMeta{
+		OPID:        b.opid,
 		Name:        bcp.Name,
 		StartTS:     time.Now().Unix(),
 		Compression: bcp.Compression,
@@ -82,9 +85,7 @@ func (b *Backup) run(bcp pbm.BackupCmd) (err error) {
 	}
 
 	rsName := inf.SetName
-	if rsName == "" {
-		rsName = pbm.NoReplset
-	}
+
 	rsMeta := pbm.BackupReplset{
 		Name:         rsName,
 		OplogName:    getDstName("oplog", bcp, inf.SetName),
@@ -95,11 +96,10 @@ func (b *Backup) run(bcp pbm.BackupCmd) (err error) {
 		FirstWriteTS: primitive.Timestamp{T: 1, I: 1},
 	}
 
-	stg, err := b.cn.GetStorage()
+	stg, err := b.cn.GetStorage(l)
 	if err != nil {
 		return errors.Wrap(err, "unable to get PBM storage configuration settings")
 	}
-
 	// on any error the RS' and the backup' (in case this is the backup leader) meta will be marked aproprietly
 	defer func() {
 		if err != nil {
@@ -117,7 +117,7 @@ func (b *Backup) run(bcp pbm.BackupCmd) (err error) {
 
 				meta.Status = pbm.StatusCancelled
 				meta.Replsets = append(meta.Replsets, rsMeta)
-				b.node.Log.Info(pbm.CmdBackup, bcp.Name, "delete artefacts from storage: %v", b.cn.DeleteBackupFiles(meta, stg))
+				l.Info("delete artefacts from storage: %v", b.cn.DeleteBackupFiles(meta, stg))
 			}
 
 			// protection from the lagging node.
@@ -126,11 +126,11 @@ func (b *Backup) run(bcp pbm.BackupCmd) (err error) {
 			// this is what we want to prevent. see https://jira.percona.com/browse/PBM-520
 			if meta.Status != pbm.StatusStarting {
 				ferr := b.cn.ChangeRSState(bcp.Name, rsMeta.Name, status, err.Error())
-				b.node.Log.Info(pbm.CmdBackup, bcp.Name, "mark RS as %s `%v`: %v", status, err, ferr)
+				l.Info("mark RS as %s `%v`: %v", status, err, ferr)
 			}
 			if inf.IsLeader() {
 				ferr := b.cn.ChangeBackupState(bcp.Name, status, err.Error())
-				b.node.Log.Info(pbm.CmdBackup, bcp.Name, "mark backup as %s `%v`: %v", status, err, ferr)
+				l.Info("mark backup as %s `%v`: %v", status, err, ferr)
 			}
 		}
 	}()
@@ -166,7 +166,7 @@ func (b *Backup) run(bcp pbm.BackupCmd) (err error) {
 				case <-tk.C:
 					err := b.cn.BackupHB(bcp.Name)
 					if err != nil {
-						b.node.Log.Error(pbm.CmdBackup, bcp.Name, "send pbm heartbeat: %v", err)
+						l.Error("send pbm heartbeat: %v", err)
 					}
 				case <-hbstop:
 					return
@@ -234,12 +234,12 @@ func (b *Backup) run(bcp pbm.BackupCmd) (err error) {
 		sz *= 4
 	}
 
-	dump := newDump(b.node.ConnURI(), runtime.NumCPU()/2)
+	dump := newDump(b.node.ConnURI(), b.node.DumpConns())
 	_, err = Upload(b.ctx, dump, stg, bcp.Compression, rsMeta.DumpName, sz)
 	if err != nil {
 		return errors.Wrap(err, "mongodump")
 	}
-	b.node.Log.Info(pbm.CmdBackup, bcp.Name, "mongodump finished, waiting for the oplog")
+	l.Info("mongodump finished, waiting for the oplog")
 
 	err = b.cn.ChangeRSState(bcp.Name, rsMeta.Name, pbm.StatusDumpDone, "")
 	if err != nil {
@@ -403,17 +403,19 @@ func Upload(ctx context.Context, src Source, dst storage.Storage, compression pb
 	case <-saveDone:
 	}
 
+	r.Close()
+
 	if !err.nil() {
 		return 0, err
 	}
 
-	return n, r.Close()
+	return n, nil
 }
 
 func (b *Backup) reconcileStatus(bcpName string, status pbm.Status, ninf *pbm.NodeInfo, timeout *time.Duration) error {
 	shards := []pbm.Shard{
 		{
-			ID:   ninf.SetName,
+			RS:   ninf.SetName,
 			Host: ninf.SetName + "/" + strings.Join(ninf.Hosts, ","),
 		},
 	}
@@ -492,12 +494,12 @@ func (b *Backup) converged(bcpName string, shards []pbm.Shard, status pbm.Status
 
 	for _, sh := range shards {
 		for _, shard := range bmeta.Replsets {
-			if shard.Name == sh.ID {
+			if shard.Name == sh.RS {
 				// check if node alive
 				lock, err := b.cn.GetLockData(&pbm.LockHeader{
-					Type:       pbm.CmdBackup,
-					BackupName: bcpName,
-					Replset:    shard.Name,
+					Type:    pbm.CmdBackup,
+					OPID:    b.opid,
+					Replset: shard.Name,
 				})
 
 				// nodes are cleaning its locks moving to the done status
