@@ -8,10 +8,12 @@ import (
 
 	"github.com/pkg/errors"
 	"go.mongodb.org/mongo-driver/bson"
+	"go.mongodb.org/mongo-driver/bson/primitive"
 	"go.mongodb.org/mongo-driver/mongo"
 	"go.mongodb.org/mongo-driver/mongo/options"
 	"gopkg.in/yaml.v2"
 
+	"github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
 	"github.com/percona/percona-backup-mongodb/pbm/storage/blackhole"
 	"github.com/percona/percona-backup-mongodb/pbm/storage/fs"
@@ -20,15 +22,15 @@ import (
 
 // Config is a pbm config
 type Config struct {
-	PITR    PITRConf    `bson:"pitr" json:"pitr" yaml:"pitr"`
-	Storage StorageConf `bson:"storage" json:"storage" yaml:"storage"`
-	Restore RestoreConf `bson:"restore" json:"restore,omitempty" yaml:"restore,omitempty"`
+	PITR    PITRConf            `bson:"pitr" json:"pitr" yaml:"pitr"`
+	Storage StorageConf         `bson:"storage" json:"storage" yaml:"storage"`
+	Restore RestoreConf         `bson:"restore" json:"restore,omitempty" yaml:"restore,omitempty"`
+	Epoch   primitive.Timestamp `bson:"epoch" json:"-" yaml:"-"`
 }
 
 // PITRConf is a Point-In-Time Recovery options
 type PITRConf struct {
-	Enabled bool  `bson:"enabled" json:"enabled" yaml:"enabled"`
-	Changed int64 `bson:"changed" json:"-" yaml:"-"`
+	Enabled bool `bson:"enabled" json:"enabled" yaml:"enabled"`
 }
 
 // StorageType represents a type of the destination storage for backups
@@ -56,7 +58,7 @@ type RestoreConf struct {
 
 type confMap map[string]reflect.Kind
 
-// _confmap is list valid keys and its types of current config
+// _confmap is a list of config's valid keys and its types
 var _confmap confMap
 
 func init() {
@@ -100,10 +102,18 @@ func (p *PBM) SetConfig(cfg Config) error {
 			return errors.Wrap(err, "cast storage")
 		}
 	}
+	ct, err := p.ClusterTime()
+	if err != nil {
+		return errors.Wrap(err, "get cluster time")
+	}
 
-	cfg.PITR.Changed = time.Now().Unix()
+	cfg.Epoch = ct
 
-	_, err := p.Conn.Database(DB).Collection(ConfigCollection).UpdateOne(
+	// TODO: if store or pitr changed - need to bump epoch
+	// TODO: struct tags to config opts `pbm:"resync,epoch"`?
+	p.GetConfig()
+
+	_, err = p.Conn.Database(DB).Collection(ConfigCollection).UpdateOne(
 		p.ctx,
 		bson.D{},
 		bson.M{"$set": cfg},
@@ -112,16 +122,6 @@ func (p *PBM) SetConfig(cfg Config) error {
 	return errors.Wrap(err, "mongo ConfigCollection UpdateOne")
 }
 
-func (p *PBM) ConfigBumpPITRepoch() error {
-	_, err := p.Conn.Database(DB).Collection(ConfigCollection).UpdateOne(
-		p.ctx,
-		bson.D{},
-		bson.M{"$set": bson.M{"pitr.changed": time.Now().Unix()}},
-	)
-
-	return errors.Wrap(err, "write to db")
-}
-
 func (p *PBM) SetConfigVar(key, val string) error {
 	if !ValidateConfigKey(key) {
 		return errors.New("invalid config key")
@@ -166,10 +166,14 @@ func (p *PBM) SetConfigVar(key, val string) error {
 }
 
 func (p *PBM) confSetPITR(k string, v bool) error {
-	_, err := p.Conn.Database(DB).Collection(ConfigCollection).UpdateOne(
+	ct, err := p.ClusterTime()
+	if err != nil {
+		return errors.Wrap(err, "get cluster time")
+	}
+	_, err = p.Conn.Database(DB).Collection(ConfigCollection).UpdateOne(
 		p.ctx,
 		bson.D{},
-		bson.M{"$set": bson.M{k: v, "pitr.changed": time.Now().Unix()}},
+		bson.M{"$set": bson.M{k: v, "pitr.changed": time.Now().Unix(), "epoch": ct}},
 	)
 
 	return err
@@ -248,7 +252,7 @@ var ErrStorageUndefined = errors.New("storage undefined")
 
 // GetStorage reads current storage config and creates and
 // returns respective storage.Storage object
-func (p *PBM) GetStorage() (storage.Storage, error) {
+func (p *PBM) GetStorage(l *log.Event) (storage.Storage, error) {
 	c, err := p.GetConfig()
 	if err != nil {
 		return nil, errors.Wrap(err, "get config")
@@ -256,7 +260,7 @@ func (p *PBM) GetStorage() (storage.Storage, error) {
 
 	switch c.Storage.Type {
 	case StorageS3:
-		return s3.New(c.Storage.S3)
+		return s3.New(c.Storage.S3, l)
 	case StorageFilesystem:
 		return fs.New(c.Storage.Filesystem), nil
 	case StorageBlackHole:
