@@ -3,7 +3,6 @@ package pbm
 import (
 	"context"
 	"fmt"
-	"os"
 	"strings"
 
 	"github.com/pkg/errors"
@@ -13,12 +12,12 @@ import (
 )
 
 type Node struct {
-	rs   string
-	me   string
-	ctx  context.Context
-	cn   *mongo.Client
-	curi string
-	Log  *Logger
+	rs        string
+	me        string
+	ctx       context.Context
+	cn        *mongo.Client
+	curi      string
+	dumpConns int
 }
 
 // ReplRole is a replicaset role in sharded cluster
@@ -39,11 +38,11 @@ const (
 	TmpRolesCollection = `pbmRRoles`
 )
 
-func NewNode(ctx context.Context, curi string) (*Node, error) {
+func NewNode(ctx context.Context, curi string, dumpConns int) (*Node, error) {
 	n := &Node{
-		ctx:  ctx,
-		curi: curi,
-		Log:  &Logger{},
+		ctx:       ctx,
+		curi:      curi,
+		dumpConns: dumpConns,
 	}
 	err := n.Connect()
 	if err != nil {
@@ -56,16 +55,9 @@ func NewNode(ctx context.Context, curi string) (*Node, error) {
 	}
 	n.rs, n.me = nodeInfo.SetName, nodeInfo.Me
 
-	n.Log.SetOut(os.Stderr)
-
 	return n, nil
 }
 
-func (n *Node) InitLogger(cn *PBM) {
-	n.Log = NewLogger(cn, n.rs, n.me)
-	n.Log.SetOut(os.Stderr)
-}
-
 // ID returns node ID
 func (n *Node) ID() string {
 	return fmt.Sprintf("%s/%s", n.rs, n.me)
@@ -201,6 +193,10 @@ func (n *Node) ConnURI() string {
 	return n.curi
 }
 
+func (n *Node) DumpConns() int {
+	return n.dumpConns
+}
+
 func (n *Node) Session() *mongo.Client {
 	return n.cn
 }
