@@ -17,6 +17,8 @@ import (
 	"go.mongodb.org/mongo-driver/mongo/readconcern"
 	"go.mongodb.org/mongo-driver/mongo/readpref"
 	"go.mongodb.org/mongo-driver/mongo/writeconcern"
+
+	"github.com/percona/percona-backup-mongodb/pbm/log"
 )
 
 const (
@@ -47,9 +49,12 @@ const (
 	PITRChunksCollection = "pbmPITRChunks"
 	//PITRChunksOldCollection contains archived index metadata of PITR chunks
 	PITRChunksOldCollection = "pbmPITRChunks.old"
-
-	// NoReplset is the name of a virtual replica set of the standalone node
-	NoReplset = "pbmnoreplicaset"
+	// StatusCollection stores pbm status
+	StatusCollection = "pbmStatus"
+	// PBMOpLogCollection contains log of aquired locks (hence run ops)
+	PBMOpLogCollection = "pbmOpLog"
+	// AgentsStatusCollection is a agents registry with its status/health checks
+	AgentsStatusCollection = "pbmAgents"
 
 	// MetadataFileSuffix is a suffix for the metadata file on a storage
 	MetadataFileSuffix = ".pbm.json"
@@ -69,6 +74,29 @@ const (
 	CmdDeleteBackup     Command = "delete"
 )
 
+func (c Command) String() string {
+	switch c {
+	case CmdBackup:
+		return "Snapshot backup"
+	case CmdRestore:
+		return "Snapshot restore"
+	case CmdCancelBackup:
+		return "Backup cancelation"
+	case CmdResyncBackupList:
+		return "Resync storage"
+	case CmdPITR:
+		return "PITR incremental backup"
+	case CmdPITRestore:
+		return "PITR restore"
+	case CmdDeleteBackup:
+		return "Delete"
+	default:
+		return "Undefined"
+	}
+}
+
+type OPID primitive.ObjectID
+
 type Cmd struct {
 	Cmd        Command         `bson:"cmd"`
 	Backup     BackupCmd       `bson:"backup,omitempty"`
@@ -76,6 +104,25 @@ type Cmd struct {
 	PITRestore PITRestoreCmd   `bson:"pitrestore,omitempty"`
 	Delete     DeleteBackupCmd `bson:"delete,omitempty"`
 	TS         int64           `bson:"ts"`
+	OPID       OPID            `bson:"-"`
+}
+
+func OPIDfromStr(s string) (OPID, error) {
+	o, err := primitive.ObjectIDFromHex(s)
+	if err != nil {
+		return OPID(primitive.NilObjectID), err
+	}
+	return OPID(o), nil
+}
+
+func NilOPID() OPID { return OPID(primitive.NilObjectID) }
+
+func (o OPID) String() string {
+	return primitive.ObjectID(o).Hex()
+}
+
+func (o OPID) Obj() primitive.ObjectID {
+	return primitive.ObjectID(o)
 }
 
 func (c Cmd) String() string {
@@ -149,15 +196,27 @@ const (
 	CompressionTypeS2     CompressionType = "s2"
 )
 
-const PITRcheckPeriod = time.Second * 15
+const (
+	PITRcheckRange       = time.Second * 15
+	AgentsStatCheckRange = time.Second * 5
+)
 
 var (
 	WaitActionStart = time.Second * 15
-	WaitBackupStart = WaitActionStart + PITRcheckPeriod*12/10
+	WaitBackupStart = WaitActionStart + PITRcheckRange*12/10
 )
 
+// OpLog represents log of started operation.
+// Operation progress can be get from logs by OPID.
+// Basically it is a log of all ever taken locks. With the
+// uniqueness by rs + opid
+type OpLog struct {
+	LockHeader `bson:",inline" json:",inline"`
+}
+
 type PBM struct {
 	Conn *mongo.Client
+	log  *log.Logger
 	ctx  context.Context
 }
 
@@ -222,9 +281,18 @@ func New(ctx context.Context, uri, appName string) (*PBM, error) {
 	return pbm, errors.Wrap(pbm.setupNewDB(), "setup a new backups db")
 }
 
+func (p *PBM) InitLogger(rs, node string) {
+	p.log = log.New(p.Conn.Database(DB).Collection(LogCollection), rs, node)
+}
+
+func (p *PBM) Logger() *log.Logger {
+	return p.log
+}
+
 const (
-	cmdCollectionSizeBytes  = 1 << 10 * 10 // size 10kb ~ 50 commands
-	logsCollectionSizeBytes = 1 << 20      // 1Mb
+	cmdCollectionSizeBytes      = 1 << 20  // 1Mb
+	pbmOplogCollectionSizeBytes = 10 << 20 // 1Mb
+	logsCollectionSizeBytes     = 50 << 20 // 50Mb
 )
 
 // setup a new DB for PBM
@@ -279,6 +347,26 @@ func (p *PBM) setupNewDB() error {
 		return errors.Wrapf(err, "ensure lock index on %s", LockOpCollection)
 	}
 
+	err = p.Conn.Database(DB).RunCommand(
+		p.ctx,
+		bson.D{{"create", PBMOpLogCollection}, {"capped", true}, {"size", pbmOplogCollectionSizeBytes}},
+	).Err()
+	if err != nil && !strings.Contains(err.Error(), "already exists") {
+		return errors.Wrap(err, "ensure log collection")
+	}
+	_, err = p.Conn.Database(DB).Collection(PBMOpLogCollection).Indexes().CreateOne(
+		p.ctx,
+		mongo.IndexModel{
+			Keys: bson.D{{"opid", 1}, {"replset", 1}},
+			Options: options.Index().
+				SetUnique(true).
+				SetSparse(true),
+		},
+	)
+	if err != nil && !strings.Contains(err.Error(), "already exists") {
+		return errors.Wrapf(err, "ensure lock index on %s", LockOpCollection)
+	}
+
 	// create indexs for the pitr cunks
 	_, err = p.Conn.Database(DB).Collection(PITRChunksCollection).Indexes().CreateMany(
 		p.ctx,
@@ -342,6 +430,7 @@ func connect(ctx context.Context, uri, appName string) (*mongo.Client, error) {
 
 // BackupMeta is a backup's metadata
 type BackupMeta struct {
+	OPID             string              `bson:"opid" json:"opid"`
 	Name             string              `bson:"name" json:"name"`
 	Replsets         []BackupReplset     `bson:"replsets" json:"replsets"`
 	Compression      CompressionType     `bson:"compression" json:"compression"`
@@ -411,11 +500,17 @@ func (b *BackupMeta) RS(name string) *BackupReplset {
 	return nil
 }
 
+func (p *PBM) ChangeBackupStateOPID(opid string, s Status, msg string) error {
+	return p.changeBackupState(bson.D{{"opid", opid}}, s, msg)
+}
 func (p *PBM) ChangeBackupState(bcpName string, s Status, msg string) error {
+	return p.changeBackupState(bson.D{{"name", bcpName}}, s, msg)
+}
+func (p *PBM) changeBackupState(clause bson.D, s Status, msg string) error {
 	ts := time.Now().UTC().Unix()
 	_, err := p.Conn.Database(DB).Collection(BcpCollection).UpdateOne(
 		p.ctx,
-		bson.D{{"name", bcpName}},
+		clause,
 		bson.D{
 			{"$set", bson.M{"status": s}},
 			{"$set", bson.M{"last_transition_ts": ts}},
@@ -512,8 +607,16 @@ func (p *PBM) SetRSLastWrite(bcpName string, rsName string, ts primitive.Timesta
 }
 
 func (p *PBM) GetBackupMeta(name string) (*BackupMeta, error) {
+	return p.getBackupMeta(bson.D{{"name", name}})
+}
+
+func (p *PBM) GetBackupByOPID(opid string) (*BackupMeta, error) {
+	return p.getBackupMeta(bson.D{{"opid", opid}})
+}
+
+func (p *PBM) getBackupMeta(clause bson.D) (*BackupMeta, error) {
 	b := new(BackupMeta)
-	res := p.Conn.Database(DB).Collection(BcpCollection).FindOne(p.ctx, bson.D{{"name", name}})
+	res := p.Conn.Database(DB).Collection(BcpCollection).FindOne(p.ctx, clause)
 	if res.Err() != nil {
 		if res.Err() == mongo.ErrNoDocuments {
 			return b, nil
@@ -621,6 +724,13 @@ func (p *PBM) GetShards() ([]Shard, error) {
 		if err != nil {
 			return nil, errors.Wrap(err, "message decode")
 		}
+		s.RS = s.ID
+		// _id may differ from the rs name, so extract rs name from the host (format like "rs2/localhost:27017")
+		// see https://jira.percona.com/browse/PBM-595
+		h := strings.Split(s.Host, "/")
+		if len(h) > 1 {
+			s.RS = h[0]
+		}
 		shards = append(shards, s)
 	}
 
@@ -663,6 +773,43 @@ func (p *PBM) ClusterTime() (primitive.Timestamp, error) {
 	return inf.ClusterTime.ClusterTime, nil
 }
 
+func (p *PBM) LogGet(r *log.LogRequest, limit int64) ([]log.LogEntry, error) {
+	return p.log.Get(r, limit, false)
+}
+
+func (p *PBM) LogGetExactSeverity(r *log.LogRequest, limit int64) ([]log.LogEntry, error) {
+	return p.log.Get(r, limit, true)
+}
+
+type Epoch primitive.Timestamp
+
+func (p *PBM) GetEpoch() (Epoch, error) {
+	c, err := p.GetConfig()
+	if err != nil {
+		return Epoch{}, errors.Wrap(err, "get config")
+	}
+
+	return Epoch(c.Epoch), nil
+}
+
+func (p *PBM) ResetEpoch() (Epoch, error) {
+	ct, err := p.ClusterTime()
+	if err != nil {
+		return Epoch{}, errors.Wrap(err, "get cluster time")
+	}
+	_, err = p.Conn.Database(DB).Collection(ConfigCollection).UpdateOne(
+		p.ctx,
+		bson.D{},
+		bson.M{"$set": bson.M{"epoch": ct}},
+	)
+
+	return Epoch(ct), err
+}
+
+func (e Epoch) TS() primitive.Timestamp {
+	return primitive.Timestamp(e)
+}
+
 // FileCompression return compression alg based on given file extention
 func FileCompression(ext string) CompressionType {
 	switch ext {
@@ -676,3 +823,99 @@ func FileCompression(ext string) CompressionType {
 		return CompressionTypeS2
 	}
 }
+
+type AgentStat struct {
+	Node          string              `bson:"n"`
+	RS            string              `bson:"rs"`
+	Ver           string              `bson:"v"`
+	PBMStatus     SubsysStatus        `bson:"pbms"`
+	NodeStatus    SubsysStatus        `bson:"nodes"`
+	StorageStatus SubsysStatus        `bson:"stors"`
+	Heartbeat     primitive.Timestamp `bson:"hb"`
+}
+
+type SubsysStatus struct {
+	OK  bool   `bson:"ok"`
+	Err string `bson:"e"`
+}
+
+func (s *AgentStat) OK() (ok bool, errs []string) {
+	ok = true
+	if !s.PBMStatus.OK {
+		ok = false
+		errs = append(errs, fmt.Sprintf("PBM connection: %s", s.PBMStatus.Err))
+	}
+	if !s.NodeStatus.OK {
+		ok = false
+		errs = append(errs, fmt.Sprintf("node connection: %s", s.NodeStatus.Err))
+	}
+	if !s.StorageStatus.OK {
+		ok = false
+		errs = append(errs, fmt.Sprintf("storage: %s", s.StorageStatus.Err))
+	}
+
+	return ok, errs
+}
+
+func (p *PBM) SetAgentStatus(stat AgentStat) error {
+	ct, err := p.ClusterTime()
+	if err != nil {
+		return errors.Wrap(err, "get cluster time")
+	}
+	stat.Heartbeat = ct
+
+	_, err = p.Conn.Database(DB).Collection(AgentsStatusCollection).ReplaceOne(
+		p.ctx,
+		bson.D{{"n", stat.Node}, {"rs", stat.RS}},
+		stat,
+		options.Replace().SetUpsert(true),
+	)
+	return errors.Wrap(err, "write into db")
+}
+
+func (p *PBM) RmAgentStatus(stat AgentStat) error {
+	_, err := p.Conn.Database(DB).Collection(AgentsStatusCollection).DeleteOne(
+		p.ctx,
+		bson.D{{"n", stat.Node}, {"rs", stat.RS}},
+	)
+
+	return err
+}
+
+// GetAgentStatus returns agent status by given node and rs
+// it's up to user how to handle ErrNoDocuments
+func (p *PBM) GetAgentStatus(rs, node string) (s AgentStat, err error) {
+	res := p.Conn.Database(DB).Collection(AgentsStatusCollection).FindOne(
+		p.ctx,
+		bson.D{{"n", node}, {"rs", rs}},
+	)
+	if res.Err() != nil {
+		return s, errors.Wrap(res.Err(), "query mongo")
+	}
+
+	err = res.Decode(&s)
+	return s, errors.Wrap(err, "decode")
+}
+
+// AgentStatusGC cleans up stale agent statuses
+func (p *PBM) AgentStatusGC() error {
+	ct, err := p.ClusterTime()
+	if err != nil {
+		return errors.Wrap(err, "get cluster time")
+	}
+	// 30 secs is the connection time out for mongo. So if there are some connection issues the agent checker
+	// may stuck for 30 sec on ping (trying to connect), it's HB became stale and it would be collected.
+	// Which would lead to the false clamin "not found" in the status output. So stale range should at least 30 sec
+	// (+5 just in case).
+	stalesec := AgentsStatCheckRange.Seconds() * 3
+	if stalesec < 35 {
+		stalesec = 35
+	}
+	ct.T -= uint32(stalesec)
+	_, err = p.Conn.Database(DB).Collection(AgentsStatusCollection).DeleteMany(
+		p.ctx,
+		bson.M{"hb": bson.M{"$lt": ct}},
+	)
+
+	return errors.Wrap(err, "delete")
+}
