@@ -16,6 +16,7 @@ import (
 	"go.mongodb.org/mongo-driver/mongo"
 
 	"github.com/percona/percona-backup-mongodb/pbm"
+	"github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
 )
 
@@ -24,7 +25,7 @@ func init() {
 	//
 	// duplicated in backup/restore packages just
 	// in the sake of clarity
-	mlog.SetDateFormat(pbm.LogTimeFormat)
+	mlog.SetDateFormat(log.LogTimeFormat)
 	mlog.SetVerbosity(&options.Verbosity{
 		VLevel: mlog.DebugLow,
 	})
@@ -61,7 +62,8 @@ type Restore struct {
 	pitrChunks []pbm.PITRChunk
 	pitrLastTS int64
 	oplog      *Oplog
-	log        *pbm.Logger
+	log        *log.Event
+	opid       string
 }
 
 // New creates a new restore object
@@ -69,7 +71,6 @@ func New(cn *pbm.PBM, node *pbm.Node) *Restore {
 	return &Restore{
 		cn:   cn,
 		node: node,
-		log:  node.Log,
 	}
 }
 
@@ -82,19 +83,19 @@ func (r *Restore) Close() {
 }
 
 // Snapshot do the snapshot's (mongo dump) restore
-func (r *Restore) Snapshot(cmd pbm.RestoreCmd) (err error) {
+func (r *Restore) Snapshot(cmd pbm.RestoreCmd, opid pbm.OPID, l *log.Event) (err error) {
 	defer func() {
 		if err != nil {
 			ferr := r.MarkFailed(err)
 			if ferr != nil {
-				r.log.Error(pbm.CmdRestore, cmd.BackupName, "mark restore as failed `%v`: %v", err, ferr)
+				l.Error("mark restore as failed `%v`: %v", err, ferr)
 			}
 		}
 
 		r.Close()
 	}()
 
-	err = r.Init(cmd.Name)
+	err = r.init(cmd.Name, opid, l)
 	if err != nil {
 		return err
 	}
@@ -113,19 +114,19 @@ func (r *Restore) Snapshot(cmd pbm.RestoreCmd) (err error) {
 }
 
 // PITR do Point-in-Time Recovery
-func (r *Restore) PITR(cmd pbm.PITRestoreCmd) (err error) {
+func (r *Restore) PITR(cmd pbm.PITRestoreCmd, opid pbm.OPID, l *log.Event) (err error) {
 	defer func() {
 		if err != nil {
 			ferr := r.MarkFailed(err)
 			if ferr != nil {
-				r.log.Error(pbm.CmdRestore, time.Unix(cmd.TS, 0).UTC().Format(time.RFC3339), "mark restore as failed `%v`: %v", err, ferr)
+				l.Error("mark restore as failed `%v`: %v", err, ferr)
 			}
 		}
 
 		r.Close()
 	}()
 
-	err = r.Init(cmd.Name)
+	err = r.init(cmd.Name, opid, l)
 	if err != nil {
 		return err
 	}
@@ -148,7 +149,9 @@ func (r *Restore) PITR(cmd pbm.PITRestoreCmd) (err error) {
 	return r.Done()
 }
 
-func (r *Restore) Init(name string) (err error) {
+func (r *Restore) init(name string, opid pbm.OPID, l *log.Event) (err error) {
+	r.log = l
+
 	r.nodeInfo, err = r.node.GetInfo()
 	if err != nil {
 		return errors.Wrap(err, "get node data")
@@ -159,7 +162,9 @@ func (r *Restore) Init(name string) (err error) {
 
 	r.name = name
 
+	r.opid = opid.String()
 	meta := &pbm.RestoreMeta{
+		OPID:     r.opid,
 		Name:     r.name,
 		StartTS:  time.Now().Unix(),
 		Status:   pbm.StatusStarting,
@@ -179,7 +184,7 @@ func (r *Restore) Init(name string) (err error) {
 				case <-tk.C:
 					err := r.cn.RestoreHB(r.name)
 					if err != nil {
-						r.log.Error(pbm.CmdRestore, name, "send heartbeat: %v", err)
+						l.Error("send heartbeat: %v", err)
 					}
 				case <-r.stopHB:
 					return
@@ -207,7 +212,7 @@ func (r *Restore) Init(name string) (err error) {
 		return errors.Wrap(err, "add shard's metadata")
 	}
 
-	r.stg, err = r.cn.GetStorage()
+	r.stg, err = r.cn.GetStorage(r.log)
 	if err != nil {
 		return errors.Wrap(err, "get backup storage")
 	}
@@ -267,7 +272,7 @@ func (r *Restore) prepareChunks(from, to primitive.Timestamp) error {
 		}
 		nextStart = c.EndTS
 
-		err := r.stg.CheckFile(c.FName)
+		_, err := r.stg.FileStat(c.FName)
 		if err != nil {
 			return errors.Errorf("failed to ensure chunk %v.%v on the storage, file: %s, error: %v", c.StartTS, c.EndTS, c.FName, err)
 		}
@@ -319,11 +324,11 @@ func (r *Restore) prepareSnapshot() (err error) {
 		return errors.Errorf("metadata for replset/shard %s is not found", r.nodeInfo.SetName)
 	}
 
-	err = r.stg.CheckFile(r.dumpFile)
+	_, err = r.stg.FileStat(r.dumpFile)
 	if err != nil {
 		return errors.Errorf("failed to ensure snapshot file %s: %v", r.dumpFile, err)
 	}
-	err = r.stg.CheckFile(r.oplogFile)
+	_, err = r.stg.FileStat(r.oplogFile)
 	if err != nil {
 		return errors.Errorf("failed to ensure oplog file %s: %v", r.oplogFile, err)
 	}
@@ -408,7 +413,7 @@ func (r *Restore) RunSnapshot() (err error) {
 	defer func() {
 		err := r.node.DropTMPcoll()
 		if err != nil {
-			r.log.Error(pbm.CmdRestore, r.bcp.Name, "dropping tmp collections: %v", err)
+			r.log.Error("dropping tmp collections: %v", err)
 		}
 	}()
 
@@ -448,7 +453,7 @@ func (r *Restore) RunSnapshot() (err error) {
 	if err != nil {
 		return errors.Wrap(err, "set shard's StatusDumpDone")
 	}
-	r.log.Info(pbm.CmdRestore, r.bcp.Name, "mongorestore finished")
+	r.log.Info("mongorestore finished")
 
 	if r.nodeInfo.IsLeader() {
 		err = r.reconcileStatus(pbm.StatusDumpDone, nil)
@@ -462,7 +467,7 @@ func (r *Restore) RunSnapshot() (err error) {
 		return errors.Wrap(err, "waiting for start")
 	}
 
-	r.log.Info(pbm.CmdRestore, r.bcp.Name, "starting oplog replay")
+	r.log.Info("starting oplog replay")
 
 	or, err := r.stg.SourceReader(r.oplogFile)
 	if err != nil {
@@ -480,14 +485,14 @@ func (r *Restore) RunSnapshot() (err error) {
 	if err != nil {
 		return errors.Wrap(err, "oplog apply")
 	}
-	r.log.Info(pbm.CmdRestore, r.bcp.Name, "oplog replay finished on %v", lts)
+	r.log.Info("oplog replay finished on %v", lts)
 
 	cusr, err := r.node.CurrentUser()
 	if err != nil {
 		return errors.Wrap(err, "get current user")
 	}
 
-	r.log.Info(pbm.CmdRestore, r.bcp.Name, "restoring users and roles")
+	r.log.Info("restoring users and roles")
 	err = r.restoreUsers(cusr)
 	if err != nil {
 		return errors.Wrap(err, "restore users 'n' roles")
@@ -498,27 +503,25 @@ func (r *Restore) RunSnapshot() (err error) {
 
 // RestoreChunks replays PITR oplog chunks
 func (r *Restore) RestoreChunks() error {
-	n := time.Unix(r.pitrLastTS, 0).UTC().Format(time.RFC3339)
-	r.log.Info(pbm.CmdPITRestore, n, "replay chunks")
+	r.log.Info("replay chunks")
 
-	var upto int64
 	var lts primitive.Timestamp
 	var err error
 	for i, chnk := range r.pitrChunks {
 		if i == len(r.pitrChunks)-1 {
-			upto = r.pitrLastTS
+			r.oplog.SetEdgeUnix(r.pitrLastTS)
 		}
-		lts, err = r.replyChunk(chnk.FName, chnk.Compression, upto)
+		lts, err = r.replyChunk(chnk.FName, chnk.Compression)
 		if err != nil {
 			return errors.Errorf("replay chunk %v.%v: %v", chnk.StartTS.T, chnk.EndTS.T, err)
 		}
 	}
 
-	r.log.Info(pbm.CmdPITRestore, n, "oplog replay finished on %v <%d>", lts, upto)
+	r.log.Info("oplog replay finished on %v <%d>", lts, r.pitrLastTS)
 	return nil
 }
 
-func (r *Restore) replyChunk(file string, c pbm.CompressionType, upto int64) (lts primitive.Timestamp, err error) {
+func (r *Restore) replyChunk(file string, c pbm.CompressionType) (lts primitive.Timestamp, err error) {
 	or, err := r.stg.SourceReader(file)
 	if err != nil {
 		return lts, errors.Wrapf(err, "get object %s form the storage", file)
@@ -531,7 +534,6 @@ func (r *Restore) replyChunk(file string, c pbm.CompressionType, upto int64) (lt
 	}
 	defer oplogReader.Close()
 
-	r.oplog.SetEdgeUnix(upto)
 	lts, err = r.oplog.Apply(oplogReader)
 
 	return lts, errors.Wrap(err, "apply oplog for chunk")
@@ -623,7 +625,7 @@ func (r *Restore) restoreUsers(exclude *pbm.AuthInfo) error {
 func (r *Restore) reconcileStatus(status pbm.Status, timeout *time.Duration) error {
 	shards := []pbm.Shard{
 		{
-			ID:   r.nodeInfo.SetName,
+			RS:   r.nodeInfo.SetName,
 			Host: r.nodeInfo.SetName + "/" + strings.Join(r.nodeInfo.Hosts, ","),
 		},
 	}
@@ -702,12 +704,12 @@ func (r *Restore) converged(shards []pbm.Shard, status pbm.Status) (bool, error)
 
 	for _, sh := range shards {
 		for _, shard := range bmeta.Replsets {
-			if shard.Name == sh.ID {
+			if shard.Name == sh.RS {
 				// check if node alive
 				lock, err := r.cn.GetLockData(&pbm.LockHeader{
-					Type:       pbm.CmdRestore,
-					BackupName: r.name,
-					Replset:    shard.Name,
+					Type:    pbm.CmdRestore,
+					OPID:    r.opid,
+					Replset: shard.Name,
 				})
 
 				// nodes are cleaning its locks moving to the done status
