@@ -6,7 +6,6 @@ import (
 	"io/ioutil"
 	"net/http"
 	"net/url"
-	"os"
 	"path"
 	"runtime"
 	"strconv"
@@ -22,6 +21,7 @@ import (
 	"github.com/minio/minio-go"
 	"github.com/pkg/errors"
 
+	"github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
 )
 
@@ -91,9 +91,10 @@ const (
 
 type S3 struct {
 	opts Conf
+	log  *log.Event
 }
 
-func New(opts Conf) (*S3, error) {
+func New(opts Conf, l *log.Event) (*S3, error) {
 	err := opts.Cast()
 	if err != nil {
 		return nil, errors.Wrap(err, "cast options")
@@ -101,6 +102,7 @@ func New(opts Conf) (*S3, error) {
 
 	return &S3{
 		opts: opts,
+		log:  l,
 	}, nil
 }
 
@@ -152,8 +154,10 @@ func (s *S3) Save(name string, data io.Reader, sizeb int) error {
 				partSize = ps
 			}
 		}
-		// TODO: needs to be pbm.Logger
-		fmt.Fprintf(os.Stderr, "%s [INFO] s3.uploadPartSize is set to %d (~%dMb)\n", time.Now().Format("2006-01-02T15:04:05.000-0700"), partSize, partSize>>20)
+
+		if s.log != nil {
+			s.log.Info("s3.uploadPartSize is set to %d (~%dMb)", partSize, partSize>>20)
+		}
 
 		_, err = s3manager.NewUploader(awsSession, func(u *s3manager.Uploader) {
 			u.MaxUploadParts = s3manager.MaxUploadParts
@@ -281,10 +285,10 @@ func (s *S3) List(prefix string) ([]string, error) {
 	return files, nil
 }
 
-func (s *S3) CheckFile(name string) error {
+func (s *S3) FileStat(name string) (inf storage.FileInfo, err error) {
 	s3s, err := s.s3session()
 	if err != nil {
-		return errors.Wrap(err, "AWS session")
+		return inf, errors.Wrap(err, "AWS session")
 	}
 
 	h, err := s3s.HeadObject(&s3.HeadObjectInput{
@@ -292,17 +296,23 @@ func (s *S3) CheckFile(name string) error {
 		Key:    aws.String(path.Join(s.opts.Prefix, name)),
 	})
 	if err != nil {
-		return errors.Wrap(err, "get S3 object header")
+		if aerr, ok := err.(awserr.Error); ok && aerr.Code() == "NotFound" {
+			return inf, storage.ErrNotExist
+		}
+
+		return inf, errors.Wrap(err, "get S3 object header")
 	}
 
-	if aws.Int64Value(h.ContentLength) == 0 {
-		return errors.New("file empty")
+	inf.Size = aws.Int64Value(h.ContentLength)
+
+	if inf.Size == 0 {
+		return inf, errors.New("file empty")
 	}
 	if aws.BoolValue(h.DeleteMarker) {
-		return errors.New("file has delete marker")
+		return inf, errors.New("file has delete marker")
 	}
 
-	return nil
+	return inf, nil
 }
 
 type (
@@ -313,7 +323,7 @@ type (
 type partReader struct {
 	fname string
 	sess  *s3.S3
-	l     *llog
+	l     *log.Event
 	opts  *Conf
 	n     int64
 	tsize int64
@@ -322,7 +332,7 @@ type partReader struct {
 
 func (s *S3) newPartReader(fname string) *partReader {
 	return &partReader{
-		l:     &llog{os.Stderr, "2006-01-02T15:04:05.000-0700"},
+		l:     s.log,
 		buf:   make([]byte, downloadChuckSize),
 		opts:  &s.opts,
 		fname: fname,
@@ -446,7 +456,6 @@ func (s *S3) SourceReader(name string) (io.ReadCloser, error) {
 	go func() {
 		defer w.Close()
 
-		slog := &llog{os.Stderr, "2006-01-02T15:04:05.000-0700"}
 	Loop:
 		for {
 			for i := 0; i < downloadRetries; i++ {
@@ -458,21 +467,21 @@ func (s *S3) SourceReader(name string) (io.ReadCloser, error) {
 					return
 				}
 				if errors.Is(err, io.ErrClosedPipe) {
-					slog.Info("reader closed pipe, stopping download")
+					s.log.Warning("reader closed pipe, stopping download")
 					return
 				}
 
-				slog.Warning("got %v, try to reconnect in %v", err, time.Second*time.Duration(i+1))
+				s.log.Warning("got %v, try to reconnect in %v", err, time.Second*time.Duration(i+1))
 				time.Sleep(time.Second * time.Duration(i+1))
 				s3s, err := s.s3session()
 				if err != nil {
-					slog.Warning("recreate session")
+					s.log.Warning("recreate session")
 					continue
 				}
 				pr.setSession(s3s)
-				slog.Info("session recreated, resuming download")
+				s.log.Info("session recreated, resuming download")
 			}
-			slog.Error("download '%s/%s' file from S3: %v", s.opts.Bucket, name, err)
+			s.log.Error("download '%s/%s' file from S3: %v", s.opts.Bucket, name, err)
 			return
 		}
 	}()
@@ -480,27 +489,6 @@ func (s *S3) SourceReader(name string) (io.ReadCloser, error) {
 	return r, nil
 }
 
-type llog struct {
-	dst     io.Writer
-	tformat string
-}
-
-func (l *llog) print(level, msg string, args ...interface{}) {
-	fmt.Fprintf(l.dst, "%s [%s] %s\n", time.Now().Format(l.tformat), level, fmt.Sprintf(msg, args...))
-}
-
-func (l *llog) Error(msg string, args ...interface{}) {
-	l.print("ERROR", msg, args...)
-}
-
-func (l *llog) Warning(msg string, args ...interface{}) {
-	l.print("Warning", msg, args...)
-}
-
-func (l *llog) Info(msg string, args ...interface{}) {
-	l.print("INFO", msg, args...)
-}
-
 // Delete deletes given file.
 // It returns storage.ErrNotExist if a file isn't exists
 func (s *S3) Delete(name string) error {
